﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using TrieApp.model;
using TrieApp.view_model;

namespace TrieApp.controls
{
	public class WordEnd2Color : IValueConverter
	{
		static SolidColorBrush	blackBrush	= new SolidColorBrush(Color.FromRgb(0, 0, 0)),
								redBrush	= new SolidColorBrush(Color.FromRgb(0xff, 0, 0)),
								whiteBrush	= new SolidColorBrush(Color.FromRgb(0xff, 0xff, 0xff));

		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			TrieNode	node	= value as TrieNode;
			bool		param	= false;

			if(parameter != null && bool.TryParse(parameter.ToString(), out param))
			{
				if(param == true)
					return whiteBrush;
			}

			if(node == null || ! node.IsEndOfWord)
				return blackBrush;

			return redBrush;
		}


		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	public class NextNode2Visibility : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			TrieNode node = value as TrieNode;

			if (node == null || node.Next == null)
				return Visibility.Collapsed;

			return Visibility.Visible;
		}


		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	public class ChildNode2Visibility : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			TrieNode node = value as TrieNode;

			if (node == null || node.Children == null || node.Children.Count <= 0)
				return Visibility.Collapsed;

			return Visibility.Visible;
		}


		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	public class Bool2VisibilityCollapse : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			string	strValue	= value == null ? null : value.ToString();

			if (string.IsNullOrEmpty( strValue))
				return Visibility.Collapsed;

			bool	result;

			if(! bool.TryParse(strValue, out result))
				return Visibility.Collapsed;

			return result == true ? Visibility.Visible : Visibility.Collapsed;
		}


		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	public class Bool2VisibilityReverseCollapse : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			string strValue = value == null ? null : value.ToString();

			if (string.IsNullOrEmpty(strValue))
				return Visibility.Collapsed;

			bool result;

			if (!bool.TryParse(strValue, out result))
				return Visibility.Collapsed;

			return result == false ? Visibility.Visible : Visibility.Collapsed;
		}


		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	public class Null2VisibilityCollapse : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return value == null ? Visibility.Collapsed : Visibility.Visible	;
		}


		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	public class Null2VisibilityReverseCollapse : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return value == null ? Visibility.Visible : Visibility.Collapsed;
		}


		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}

	public class TreeViewLineConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			TreeViewItem	item	= value as TreeViewItem;
			if(item == null)
				return false;

			ItemsControl	ic		= ItemsControl.ItemsControlFromItemContainer(item);
			return ic.ItemContainerGenerator.IndexFromContainer(item) == ic.Items.Count - 1;
		}

		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new Exception("The method or operation is not implemented.");
		}
	}


	public class Visibility2VisibilityReverseCollapse : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			string strValue = value == null ? null : value.ToString();

			if (string.IsNullOrEmpty(strValue))
				return Visibility.Collapsed;

			Visibility	visi;

			if(! Enum.TryParse<Visibility>(strValue, true, out visi))
				return Visibility.Visible;

			return visi == Visibility.Collapsed || visi == Visibility.Hidden ? Visibility.Visible : Visibility.Collapsed;
		}


		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}


	public class TrieNodeImageConverter : IValueConverter
	{
		internal const string		keyImageHome		= "imgHome",
									keyImageSimple		= "imgSimpleNode",
									keyImageEnd			= "imgEndNode";

		static Image
				_imgHome				= App.Current.FindResource(keyImageHome)		as Image,
				_imgSimpleNode			= App.Current.FindResource(keyImageSimple)		as Image,
				_imgEndNode				= App.Current.FindResource(keyImageEnd)			as Image;


		public class DesignTimeImages
		{

			ResourceDictionary _resourceDictionary	= new ResourceDictionary();
		
			public DesignTimeImages()
			{
				init_instance();
			}

			private void init_instance()
			{
				_resourceDictionary.Add(keyImageHome,	CreateImage("images/home.png"));
				_resourceDictionary.Add(keyImageSimple, CreateImage("images/simple14x14.gif"));
				_resourceDictionary.Add(keyImageEnd,	CreateImage("images/end14x14.gif"));
			}
		
			public Image Home
			{
				get {  return this[keyImageHome]; }
			}

			public Image Simple
			{
				get { return this[keyImageSimple]; }
			}

			public Image End
			{
				get { return this[keyImageEnd]; }
			}

			public Image this[string key]
			{
				get { return _resourceDictionary[key] as Image; }
			}
		

			Image CreateImage(string imgPath)
			{
				Image	img = new Image();
				{
					var		bmp		= new BitmapImage();
					bmp.BeginInit();
					bmp.UriSource = new Uri(@"pack://application:,,,/TrieApp;component/" + imgPath, UriKind.RelativeOrAbsolute);
					bmp.EndInit();

					img.Source = bmp;
					img.Width = bmp.PixelWidth;
				}
				return img;
			}


		}

		/// returns an image source
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			TrieNode					trieNode		= value as TrieNode;
			DesignTimeImages			designImages	= null;

			if(_imgHome == null || _imgSimpleNode == null || _imgEndNode == null)
				designImages	= new DesignTimeImages();

			if (trieNode == null )
				return _imgSimpleNode == null ? designImages.Simple == null ? null : designImages.Simple.Source : _imgSimpleNode.Source;

			if (TrieVm.IsTrieRootNode(trieNode))
				return _imgHome == null ? designImages.Home == null ? null : designImages.Home.Source : _imgHome.Source;

			if(trieNode.IsEndOfWord)
				return _imgEndNode == null ? designImages.End == null ? null : designImages.End.Source : _imgEndNode.Source;

			return _imgSimpleNode == null ? designImages.Simple == null ? null : designImages.Simple.Source : _imgSimpleNode.Source;
		}

		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}


	public sealed class ListBoxBackgroundConverter : IValueConverter
	{
		static SolidColorBrush		_alternateBrush = new SolidColorBrush(Color.FromRgb(0xec, 0xff, 0xfe));	// #FFECFFFE

		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
			ListBoxItem		item	= value as ListBoxItem;

			ListBox			listbox = (item == null) ? null : ItemsControl.ItemsControlFromItemContainer(item) as ListBox;

			if (listbox == null)
				return Brushes.White;

			// Get the index of a ListViewItem
			int			index	= listbox.ItemContainerGenerator.IndexFromContainer(item);

			if (index % 2 == 0)
			{
				return _alternateBrush;
			}
			else
			{
				return Brushes.White;
			}
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}
}
