﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TrieApp.model.root_objects;

namespace TrieApp.model
{
	public class TrieNode : DoubleChainedNode<TrieNode>, IComparable
	{

		CharDictionaryItem		_dicoItem;
		bool					_isEndofWord;
		object					_value;
		TrieNodeList			_children		= new TrieNodeList();

		bool					_isExpanded		= false;

		public TrieNode() : base()
		{

		}

		public TrieNode(CharDictionaryItem character, TrieNode previous, TrieNode next) : base(previous, next)
		{
			_dicoItem	= character;
		}

		public char Character
		{
			get {  return _dicoItem == null ? '\0' : _dicoItem.Character; }
		}

		public int CharValue
		{
			get {  return _dicoItem == null ? 0 : _dicoItem.CharValue; }
		}

		public string Hex4
		{
			get {  return string.Format("0x{0:x4}", CharValue); }
		}

		public CharDictionaryItem DictionaryItem
		{
			get {  return _dicoItem; }
			set
			{
				if( value == _dicoItem)
					return;

				_dicoItem = value;
				NotifyPropertyChanged(() => DictionaryItem);
				NotifyPropertyChanged(() => Character);
				NotifyPropertyChanged(() => CharValue);
				NotifyPropertyChanged(() => Strings);
				NotifyPropertyChanged(() => Neighbours);
			}
		}


		public bool IsExpanded
		{
			get {  return _isExpanded; }
			set
			{
				if(value == _isExpanded)
					return;

				_isExpanded = value;
				NotifyPropertyChanged(() => IsExpanded);
			}
		}

		public bool IsEndOfWord
		{
			get {  return _isEndofWord; }
			set
			{
				if(value == _isEndofWord)
					return;

				_isEndofWord = value;
				NotifyPropertyChanged(() => IsEndOfWord);
				NotifyPropertyChanged(() => Strings);
			}
		}

		public object Value
		{
			get {  return _value; }
			set
			{
				if(value == _value)
					return;

				_value = value;
				NotifyPropertyChanged(() => Value);
			}
		}

		public TrieNodeList Children
		{
			get {  return _children; }
			internal set
			{
				if(value == _children || value == null)
					return;

				_children = value;
				NotifyPropertyChanged(() => Children);
			}
		}

		public int ChildrenTotalCount
		{
			get
			{
				if(_children == null)
					return 0;

				int count	= _children.Count;

				foreach(TrieNode node in _children)
					count += node.ChildrenTotalCount;

				return count;
			}
		}

		public int ChildrenEndOdWordsCount
		{
			get
			{
				if (_children == null)
					return 0;

				int count = _children.Where(item => item._isEndofWord).Count();

				foreach (TrieNode node in _children)
					count += node.ChildrenEndOdWordsCount;

				return count;
			}
		}


		public List<string> Strings
		{
			get
			{
				List<string>	list		= new List<string>();
				var				childWords	= from c in _children where(c._isEndofWord == true) select c;
				List<string>	neigbors	= Next == null ? null : Next.Strings;
				string			strThis		= this.Character.ToString();

				if(childWords != null)
				{
					string		str	= strThis;

					foreach(TrieNode n in childWords)
					{
						str	+= n.Character.ToString();

						if(n.IsEndOfWord)
							list.Add( str);
					}
				}

				foreach(TrieNode childNode in _children)
				{
					foreach(string str in childNode.Strings)
						list.Add(strThis + str);
				}
					

				return list;
			}
		}

		public List<TrieNode> Neighbours
		{
			get
			{
				List<TrieNode>	list	= new List<TrieNode>();

				if(this.Next == null)
					return list;

				list.Add(Next);
				list.AddRange(Next.Neighbours);

				return list;
			}
		}


		internal void ClearChildren()
		{
			foreach(TrieNode node in _children)
				node.ClearChildren();

			_children.Clear();
		}

		public int CompareTo(object obj)
		{
			TrieNode	other	= obj as TrieNode;

			if(other == null)
				return 1;

			return string.Compare(this.Character.ToString(), other.Character.ToString());
		}

		internal void Sort()
		{
			if(_children != null)
			{
				_children.SortNodes();
			}
		}
	}

	public class TrieNodeDesignTime : TrieNode
	{

		public TrieNodeDesignTime()
		{
			this.DictionaryItem	= Trie.Instance.Dictionary.LastItem;	//['z'];

			InitChildren();
		}


		void InitChildren()
		{
			//Children.AddTail(Trie.Instance.Dictionary['o']);
			//Children.AddTail(Trie.Instance.Dictionary['a']);

			//this.Next	= Trie.Instance.RootNodes['c'];
		}

	}
}
