﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using isosoft.root;

namespace TrieApp.model
{
	public class TrieNodeList : XObjectListNotifier<TrieNode>
	{

		public TrieNodeList() : base()
		{

		}

		public TrieNode this[char character]
		{
			get { return this.FirstOrDefault(c => c.Character == character); }
		}

		public TrieNode this[char character, bool isEndOfWord]
		{
			get { return this.FirstOrDefault(c => c.Character == character && c.IsEndOfWord == isEndOfWord); }
		}


		public TrieNode AddTail(TrieNode node, bool notifyChanges)
		{
			if(node == null)
				return null;

			int			count	= this.Count;
			TrieNode	last	= this.LastNode;

			node.Previous	= last;
			// node.Next		= null;

			if(last != null)
				last.Next	= node;

			this.Add(node);
			if(notifyChanges)
				NotifyCollectionChanged(this);
			return node;
		}

		public TrieNode AddTail(CharDictionaryItem c, bool notifyChanges)
		{
			if(c == null)
				return null;

			TrieNode	exists	= this[c.Character];

			if(exists == null)
				return AddTail(new TrieNode(c, null, null), notifyChanges);
			else
			{
				TrieNode	last	= this.LastNode;

				exists.Previous	= last;

				if (last != null)
					last.Next = exists;
			}

			return exists;
		}

		public void SortNodes()
		{
			base.Sort();
			foreach(TrieNode node in this)
				node.Sort();
		}

		public TrieNode LastNode
		{
			get {  return this.Count <= 0 ? null : this[this.Count -1]; }
		}
	}
}
