﻿using isosoft.root;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Windows.Threading;
using TrieApp.model;

namespace TrieApp.view_model
{
	public class TrieVm : XObjectNotifier
	{
		string			_sample_file_name		= "trie-definition-text.txt";
		string			_sourceString			= "";

		protected Trie	_trie				= Trie.Instance;
		string			_startsWith,
						_contains;
		int				_minWordLength		= 3;


		public TrieVm() : base()
		{
		}


		public string StartsWith
		{
			get { return _startsWith; }
			set
			{
				if( value == _startsWith)
					return;

				_startsWith = value;
				NotifyPropertyChanged(() => StartsWith);
				NotifyPropertyChanged(() => StringsStartingWith);
			}
		}

		public string Contains
		{
			get { return _contains; }
			set
			{
				if( value == _contains)
					return;

				_contains = value;
				NotifyPropertyChanged(() => Contains);
				NotifyPropertyChanged(() => StringsContaining);
			}
		}

		public int MinWordLenght
		{
			get { return _minWordLength; }
			set
			{
				if (value == _minWordLength)
					return;

				_minWordLength = value;
				NotifyPropertyChanged(() => MinWordLenght);
			}
		}

		public string SourceString
		{
			get {  return _sourceString; }
			set
			{
				if(value == _sourceString)
					return;

				_sourceString = value;
				NotifyPropertyChanged(() => SourceString);
			}
		}


		public string SampleFileName
		{
			get {  return _sample_file_name; }
			set
			{
				if(value == _sample_file_name)
					return;
				_sample_file_name = value;
				NotifyPropertyChanged(() => SampleFileName);
			}
		}


		public List<string> StringsStartingWith
		{
			get {  return _trie.StartsWith(_startsWith); }
		}

		public List<string> StringsContaining
		{
			get { return _trie.Containing(_contains); }
		}

		public List<string> AllStrings
		{
			get { return _trie.AllStrings; } //.OrderBy(s => s).ToList(); }
		}


		public int? StringsCount
		{
			get {  return this.LoadFileRunning ? null : (int?) AllStrings.Count; }
		}

		internal bool RaisePropertyChanged()
		{
			// curiously: this raises all property changed!

			Dispatcher.CurrentDispatcher.Invoke(() => NotifyPropertyChanged(null));

			// 150217 DispatcherHelper.CheckBeginInvokeOnUI(() => NotifyPropertyChanged(null));
			return true;
		}

		ICommand			_loadSampleCommand;
		protected bool		_loadFileCommandRunning		= false;


		public virtual bool LoadFileRunning
		{
			get {  return _loadFileCommandRunning; }
			set
			{
				if(value == _loadFileCommandRunning)
					return;

				_loadFileCommandRunning	= value;
				NotifyPropertyChanged(() => LoadFileRunning);
				NotifyPropertyChanged(() => CanRunLoadSampleFile);
				NotifyPropertyChanged(() => StringsCount);
			}
		}

		public bool CanRunLoadSampleFile
		{
			get {  return _loadFileCommandRunning ? false : true; }
		}

		void SetFileLoadingStatus(bool value)
		{
			LoadFileRunning = value;
		}

		public ICommand	LoadSampleTextCommand
		{
			get
			{
				if(_loadSampleCommand == null)
				{
					_loadSampleCommand = new CommandExecuter(() =>
						{
							Dispatcher.CurrentDispatcher.Invoke(() =>
								{
							LoadFileRunning = true;
							LoadSampleFile();
								});
						}
						, CanLoadSampleFile, false, Dispatcher.CurrentDispatcher, null);
				}
				return _loadSampleCommand;
			}
		}


		protected void AddBuiltInWords(bool notifyChanges)
		{
			List<string>		list	= new List<string>()
			{
				"tree",
				"trie",
				"try",
				"trying",
				"tea",
				"team",
			};

			_trie.ClearAll( notifyChanges);

			foreach(string str in list)
				_trie.AddString(str, null, false);

			string		sourceString	= "";

			foreach(string str in list)
				sourceString	+= str + " ";

			SourceString	= sourceString;
		}

		private async void LoadSampleFile()
		{
			StartsWith		= "";
			Contains		= "";
			//NotifyPropertyChanged(() => AllStrings);
			//NotifyPropertyChanged(() => Root);

			if(! File.Exists(_sample_file_name))
			{
				AddBuiltInWords(false);
				return;
			}

			await Dispatcher.CurrentDispatcher.Invoke( async() =>
			{
				TextReader	stream	= File.OpenText(_sample_file_name);		// OpenText reads file as UTF8
				string		str		= await stream.ReadToEndAsync();

				stream.Close();

				SourceString	= str;

				int		nodesCount = await _trie.LoadFromStringAsync(str, _minWordLength, false);

				_trie.Sort();
			});

			Dispatcher.CurrentDispatcher.Invoke(() =>
			{
				LoadFileRunning = false;
				RaisePropertyChanged();
			});
		}

		public bool CanLoadSampleFile()
		{
			return CanRunLoadSampleFile;
		}

		public List<CharDictionaryItem> Dictionary
		{
			get { return _trie.Dictionary.OrderBy( i => i.Character.ToString()).ToList(); }
		}

		public List<TrieNode> Root
		{
			get { return _trie.Root; }
		}

		public List<List<TrieNode>> NodesByDepthLevel
		{
			get {  return _trie.NodesByDepthLevel; }
		}


		public static bool IsTrieRootNode(TrieNode node)
		{
			CharDictionaryItem dico	= node == null ? null : node.DictionaryItem;

			if(dico != null && dico.Next == null && dico.Previous == null)
				return true;

			return false;
		}

	}

	public class TrieVMDesignTime : TrieVm
	{
		public TrieVMDesignTime() : base()
		{
			AddBuiltInWords(false);
			_loadFileCommandRunning = true;
		}

		public override bool LoadFileRunning
		{
			get
			{
				return _loadFileCommandRunning;
			}
			set
			{
				base.LoadFileRunning = value;
			}
		}

	}
}
